;; Generated package description from svg-tag-mode.el  -*- no-byte-compile: t -*-
(define-package "svg-tag-mode" "0.3.2.0.20240429.105240" "Replace keywords with SVG tags" '((emacs "27.1") (svg-lib "0.2")) :commit "a152bc90a7c9dc17112893a19ddf91078b909057" :authors '(("Nicolas P. Rougier" . "Nicolas.Rougier@inria.fr")) :maintainer '("Nicolas P. Rougier" . "Nicolas.Rougier@inria.fr") :keywords '("convenience") :url "https://github.com/rougier/svg-tag-mode")
